@@ -28,6 +28,7 @@ from winsdk.windows.applicationmodel.background import (
     BackgroundTaskRegistration,
     BackgroundTaskBuilder,
     BackgroundExecutionManager,
+    BackgroundAccessStatus,
     ToastNotificationActionTrigger,
 )
 from winsdk.windows.foundation import IPropertyValue, PropertyType
@@ -71,10 +72,14 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
         super().__init__(app_name, app_icon, notification_limit)
         self._appid = CoreApplication.get_id()
         self.manager = ToastNotificationManager.get_default()
-        self.notifier = self.manager.create_toast_notifier(self._appid)
-
-        # BackgroundExecutionManager.remove_access(self._appid)
-        # CoreApplication.add_background_activated(self._on_background_activated)
+        self.notifier: ToastNotification | None = None
+        if self._appid == "":
+            logger.warning(
+                "Only applications can send desktop notifications. "
+                "Could not find App ID for process."
+            )
+        else:
+            self.notifier = self.manager.create_toast_notifier(self._appid)
 
     async def request_authorisation(self) -> bool:
         """
@@ -82,24 +87,44 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
 
         :returns: Whether authorisation has been granted.
         """
+        if not await self._request_background_task_access():
+            return False
         return await self.has_authorisation()
 
     async def has_authorisation(self) -> bool:
         """
         Whether we have authorisation to send notifications.
         """
-        return self.notifier.setting == NotificationSetting.ENABLED
+        if not self.notifier:
+            return False
+        return (
+            self.notifier.setting == NotificationSetting.ENABLED
+            and await self._has_background_task_access()
+        )
 
-    async def _request_background_task_access(self) -> None:
+    async def _has_background_task_access(self) -> bool:
+        res = await BackgroundExecutionManager.request_access_async(self._appid)
+        return res not in {
+            BackgroundAccessStatus.DENIED_BY_SYSTEM_POLICY,
+            BackgroundAccessStatus.DENIED_BY_USER,
+        }
+
+    def _has_registered_background_task(self) -> bool:
+        tasks = BackgroundTaskRegistration.get_all_tasks()
+        return any(t.name == self.background_task_name for t in tasks.values())
+
+    async def _request_background_task_access(self) -> bool:
         """Request permission to activate in the background."""
+        if not self.notifier:
+            return False
+
+        if not await self._has_background_task_access():
+            logger.warning("This app is not allowed to run background tasks.")
+            return False
 
         # If background task is already registered, do nothing.
-        tasks = BackgroundTaskRegistration.get_all_tasks()
-        if any(t.value.name == self.background_task_name for t in tasks):
-            return
-
-        # Otherwise request access.
-        await BackgroundExecutionManager.request_access_async(self._appid)
+        if self._has_registered_background_task():
+            return True
 
         # Create the background tasks.
         builder = BackgroundTaskBuilder()
@@ -108,6 +133,8 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
         builder.set_trigger(ToastNotificationActionTrigger())
         builder.register()
 
+        return True
+
     async def _send(
         self,
         notification: Notification,
@@ -119,14 +146,14 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
         :param notification: Notification to send.
         :param notification_to_replace: Notification to replace, if any.
         """
+        if not self.notifier:
+            raise RuntimeError("Could not get toast notifier")
 
         if notification_to_replace:
             platform_nid = cast(str, notification_to_replace.identifier)
         else:
             platform_nid = str(uuid.uuid4())
 
-        await self._request_background_task_access()
-
         # Create notification XML.
         toast_xml = Element("toast", {"launch": "default"})
         visual_xml = SubElement(toast_xml, "visual")
@@ -208,7 +235,7 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
 
         def on_activated(sender, boxed_activated_args):
             activated_args = ToastActivatedEventArgs._from(boxed_activated_args)
-            action_id = activated_args.arguments
+            action_id = cast(str, activated_args.arguments)
 
             if action_id == "default":
                 if notification.on_clicked:
@@ -217,10 +244,10 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
             elif action_id == "action=reply&amp":
                 if notification.reply_field.on_replied:
                     boxed_text = activated_args.user_input["textBox"]
-                    text = _unbox_winrt(boxed_text)
+                    text = unbox_winrt(boxed_text)
                     notification.reply_field.on_replied(text)
 
-            else:
+            elif action_id.isnumeric():
                 action_number = int(action_id)
                 notification.buttons[action_number].on_pressed()
 
@@ -259,7 +286,7 @@ class WinRTDesktopNotifier(DesktopNotifierBase):
         self.manager.history.clear(self._appid)
 
 
-def _unbox_winrt(boxed_value):
+def unbox_winrt(boxed_value):
     """
     Unbox winrt object. See https://github.com/pywinrt/pywinrt/issues/8.
     """
@@ -268,15 +295,15 @@ def _unbox_winrt(boxed_value):
 
     value = IPropertyValue._from(boxed_value)
 
-    if value.type is PropertyType.EMPTY.value:
+    if value.type is PropertyType.EMPTY:
         return None
-    elif value.type is PropertyType.UINT8.value:
+    elif value.type is PropertyType.UINT8:
         return value.get_uint8()
-    elif value.type is PropertyType.INT16.value:
+    elif value.type is PropertyType.INT16:
         return value.get_int16()
-    elif value.type is PropertyType.UINT16.value:
+    elif value.type is PropertyType.UINT16:
         return value.get_uint16()
-    elif value.type is PropertyType.STRING.value:
+    elif value.type is PropertyType.STRING:
         return value.get_string()
     else:
         raise NotImplementedError(f"Unboxing {value.type} is not yet supported")
